#ifdef _VERTEX_

void main()
{
    gl_Position = ftransform();
    gl_TexCoord[0] = gl_MultiTexCoord0;
}

#else

float   HeightMapClouds(vec3 point)
{
    float zone = cos(point.y);
    float ang = zone * twistMagn;
    float weight = 1.0;
    float offset = 0.0;
    vec3  twistedPoint = point;

    // Compute cyclons
    vec3  cycloneCenter;
    float cycloneRadius = 1.0;
    float cycloneAmpl;
    if (tidalLock != 0.0)
    {
        cycloneCenter = vec3(0.0, 1.0, 0.0);
        cycloneRadius = length(cycloneCenter - point) / cycloneSqrtDensity;
        cycloneAmpl   = -cycloneMagn * sign(cycloneCenter.y);
    }
    else
    {
        vec4 cell = Cell2NoiseVec(point * cycloneFreq);
        cycloneCenter = cell.xyz;
        cycloneRadius = length(cycloneCenter - point) / cycloneSqrtDensity;
        cycloneAmpl   = 0.4*pi * sign(cycloneCenter.y);
    }
    if (cycloneRadius < 1.0)
    {
        float dist = 1.0 - cycloneRadius;
        float fi = mix(log(cycloneRadius), pow(dist, 3.0), cycloneRadius);
        twistedPoint = Rotate(cycloneAmpl*fi, cycloneCenter, point);
        if (cycloneRadius < 0.025)
            weight = 1.0 - pow(1.0 - cycloneRadius / 0.025, 2);
        else
            weight = 1.0;
        weight = weight * (1.0 + dist);
        //offset += 0.7 * dist;
    }

    // Compute Coriolis effect
    float sina = sin(ang);
    float cosa = cos(ang);
    twistedPoint = vec3(cosa*twistedPoint.x-sina*twistedPoint.z, twistedPoint.y, sina*twistedPoint.x+cosa*twistedPoint.z);
    twistedPoint = twistedPoint * cloudsFreq + Randomize;

    // Compute flow-like distortion
    noiseLacunarity = 3.0;
    noiseOctaves = 10;
    vec3  distort = FbmVec3(twistedPoint * 0.55) * 0.5;
    noiseOctaves = 6;
    float global = (Fbm(twistedPoint + distort) + 1.0) * 0.8;
    global = (global + offset) * weight;

    // Compute turbilence features
    noiseOctaves = cloudsOctaves;
    float turbulence = Fbm(point * 100.0 * cloudsFreq + Randomize) * 0.1;

    return  (global + turbulence * step(0.1, global) + cloudsCoverage);
}

void main()
{
    vec3  point = GetSurfacePoint();
    float height = HeightMapClouds(point);
    gl_FragColor = vec4(height);
}

#endif
